package robotserver;

import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.ServerSocket;
import java.net.Socket;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import nl.tue.id.creapro.admoveo.AdMoVeo;
import processing.core.PApplet;
import javax.swing.JTextArea;
import java.awt.BorderLayout;
import javax.swing.JScrollPane;

public class RobotServerTemplate extends PApplet {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private JFrame jFrame = null; // @jve:decl-index=0:visual-constraint="7,-3"
	private JPanel jContentPane = null;
	private JButton jExitButton = null;
	private JTextField jTextField = null;

	private final int portNumber = 5050;
	private AdMoVeo admoveo;
	private RobotHandler rHandler = null;
	private JPanel jPanel = null;
	private JScrollPane jScrollPane = null;
	private JTextArea jTextArea = null;

	public RobotServerTemplate() {
		rHandler = new RobotHandler();
		admoveo = new AdMoVeo(this,"This needs to be COMX according to your hardware...");
		rHandler.start();
	}

	/**
	 * This method initializes jFrame
	 * 
	 * @return javax.swing.JFrame
	 */
	private JFrame getJFrame() {
		if (jFrame == null) {
			jFrame = new JFrame();
			jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			jFrame.setSize(328, 236);
			jFrame.setContentPane(getJContentPane());
			jFrame.setTitle("Robot server");
		}
		return jFrame;
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJExitButton(), BorderLayout.SOUTH);
			jContentPane.add(getJPanel(), BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jExitButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJExitButton() {
		if (jExitButton == null) {
			jExitButton = new JButton();
			jExitButton.setText("Exit");
			jExitButton.setSelected(true);
			jExitButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					// close the inputstream and connection
					if (rHandler != null) rHandler.shutDown();
					System.exit(0);
				}
			});
		}
		return jExitButton;
	}

	/**
	 * This method initializes jTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getJTextField() {
		if (jTextField == null) {
			jTextField = new JTextField();
			jTextField.setPreferredSize(new Dimension(250, 20));
			jTextField.setText("Disconnected");
		}
		return jTextField;
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new BorderLayout());
			jPanel.add(getJTextField(), BorderLayout.SOUTH);
			jPanel.add(getJScrollPane(), BorderLayout.CENTER);
		}
		return jPanel;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			jScrollPane.setViewportView(getJTextArea());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jTextArea
	 * 
	 * @return javax.swing.JTextArea
	 */
	private JTextArea getJTextArea() {
		if (jTextArea == null) {
			jTextArea = new JTextArea();
		}
		return jTextArea;
	}

	/**
	 * Launches this application
	 */
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				RobotServerTemplate application = new RobotServerTemplate();
				application.getJFrame().setVisible(true);
			}
		});
	}

	class RobotHandler extends Thread {
		Socket clientSocket;
		ServerSocket serverSocket;
		BufferedReader br;
		boolean connected = false;

		public RobotHandler() {
			try {
				serverSocket = new ServerSocket(portNumber);
			} catch (java.io.IOException e) {
				System.err.println("Could not create server socket.");
				System.exit(1);
			}
		}
		
		public void shutDown() {
			connected = false;
			try {
				if (br != null) br.close();
				if (clientSocket != null) clientSocket.close();
				if (serverSocket != null) serverSocket.close();
			} catch (IOException e) {
				// e.printStackTrace();
			}
		}

		public void run() {

			while (true) {
				try {
					System.err.println("RobotServer waiting on port "
							+ serverSocket.getLocalPort());
					clientSocket = serverSocket.accept();
				} catch (java.io.IOException e) {
					System.err.println("Could not create client socket.");
					return;
				}

				try {
					br = new BufferedReader(new InputStreamReader(
							clientSocket.getInputStream()));
					connected = true;
				} catch (IOException e) {
					System.err.println("Could not create inputstream");
					return;
				}

				getJTextField().setText("Connected");

				while (connected) {
					/* 
					 * 
					 * fill in what has to be done here...
					 * 
					 */
				}
				
				getJTextField().setText("Disconnected");
			}
		}

		private String Receive() {
			try {
				return br.readLine();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				// something wrong... do as if nothing received for now
				return null;
			}
		}
	}
}
